@@ -1,20 +1,20 @@
 /* cast5.c  -  CAST5 cipher (RFC2144)
- *	Copyright (C) 1998, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
  * Libgcrypt is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Lesser general Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
  *
  * Libgcrypt is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
@@ -591,7 +591,7 @@ do_cast_setkey( CAST5_context *c, byte *key, unsigned keylen )
 	initialized = 1;
 	selftest_failed = selftest();
 	if( selftest_failed )
-	    fprintf(stderr,"CAST5 selftest failed (%s).\n", selftest_failed );
+	    log_error ("CAST5 selftest failed (%s).\n", selftest_failed );
     }
     if( selftest_failed )
 	return GCRYERR_SELFTEST;
