@@ -1,24 +1,23 @@
 /* random.c  -	random number generator
- *	Copyright (C) 1998, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
  * Libgcrypt is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
  *
  * Libgcrypt is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-
 /****************
  * This random number generator is modelled after the one described
  * in Peter Gutmann's Paper: "Software Generation of Practically
@@ -55,7 +54,8 @@
 #include "random.h"
 #include "rand-internal.h"
 #include "dynload.h"
-
+#include "cipher.h" /* only used for the rmd160_hash_buffer() prototype */
+#include "ath.h"
 
 #ifndef RAND_MAX   /* for SunOS */
   #define RAND_MAX 32767
@@ -98,15 +98,21 @@ static int did_initial_extra_seeding;
 static char *seed_file_name;
 static int allow_seed_file_update;
 
+static unsigned char failsafe_digest[DIGESTLEN];
+static int failsafe_digest_valid;
+
 static int secure_alloc;
 static int quick_test;
 static int faked_rng;
 
+static ath_mutex_t pool_lock = ATH_MUTEX_INITIALIZER;
+static int pool_is_locked; /* only used for assertion */
 
 static byte *get_random_bytes( size_t nbytes, int level, int secure );
 static void read_pool( byte *buffer, size_t length, int level );
 static void add_randomness( const void *buffer, size_t length, int source );
 static void random_poll(void);
+static void do_fast_random_poll (void);
 static void read_random_source( int requester, size_t length, int level);
 static int gather_faked( void (*add)(const void*, size_t, int), int requester,
 						    size_t length, int level );
@@ -124,20 +130,41 @@ static struct {
     ulong naddbytes;
 } rndstats;
 
+
+/* Note, we assume that this function is used before any concurrent
+   access happens */
 static void
 initialize(void)
 {
-    /* The data buffer is allocated somewhat larger, so that
-     * we can use this extra space (which is allocated in secure memory)
-     * as a temporary hash buffer */
-    rndpool = secure_alloc ? gcry_xcalloc_secure(1,POOLSIZE+BLOCKLEN)
-			   : gcry_xcalloc(1,POOLSIZE+BLOCKLEN);
-    keypool = secure_alloc ? gcry_xcalloc_secure(1,POOLSIZE+BLOCKLEN)
-			   : gcry_xcalloc(1,POOLSIZE+BLOCKLEN);
-    is_initialized = 1;
-    _gcry_cipher_modules_constructor();
+  int err;
+
+  err = ath_mutex_init (&pool_lock);
+  if (err)
+    log_fatal ("failed to create the pool lock: %s\n", strerror (err) );
+    
+  /* The data buffer is allocated somewhat larger, so that we can use
+    this extra space (which is allocated in secure memory) as a
+    temporary hash buffer */
+  rndpool = secure_alloc ? gcry_xcalloc_secure(1,POOLSIZE+BLOCKLEN)
+                         : gcry_xcalloc(1,POOLSIZE+BLOCKLEN);
+  keypool = secure_alloc ? gcry_xcalloc_secure(1,POOLSIZE+BLOCKLEN)
+                         : gcry_xcalloc(1,POOLSIZE+BLOCKLEN);
+  is_initialized = 1;
+  _gcry_cipher_modules_constructor ();
 }
 
+/* Initialize this random subsystem.  This function memrely calls the
+   initialzies and does not do anything more.  Doing this is not
+   really required but when running in a threaded environment we might
+   get a race condition otherwise. */
+void
+_gcry_random_initialize ()
+{
+  if (!is_initialized)
+    initialize ();
+}
+
+
 static void
 burn_stack (int bytes)
 {
@@ -153,7 +180,7 @@ burn_stack (int bytes)
 void
 _gcry_random_dump_stats()
 {
-    fprintf(stderr,
+    log_info (
 	    "random usage: poolsize=%d mixed=%lu polls=%lu/%lu added=%lu/%lu\n"
 	    "              outmix=%lu getlvl1=%lu/%lu getlvl2=%lu/%lu\n",
 	POOLSIZE, rndstats.mixrnd, rndstats.slowpolls, rndstats.fastpolls,
@@ -174,6 +201,9 @@ _gcry_quick_random_gen( int onoff )
 {
     int last;
 
+    /* No need to lock it here because we are only initializing.  A
+       prerequisite of the entire code is that it has already been
+       initialized before any possible concurrent access */
     read_random_source(0,0,0); /* init */
     last = quick_test;
     if( onoff != -1 )
@@ -181,21 +211,6 @@ _gcry_quick_random_gen( int onoff )
     return faked_rng? 1 : last;
 }
 
-
-/****************
- * Fill the buffer with LENGTH bytes of cryptographically strong
- * random bytes. level 0 is not very strong, 1 is strong enough
- * for most usage, 2 is good for key generation stuff but may be very slow.
- */
-void
-gcry_randomize( byte *buffer, size_t length, enum gcry_random_level level )
-{
-    char *p = get_random_bytes( length, level, 1 );
-    memcpy( buffer, p, length );
-    gcry_free(p);
-}
-
-
 int
 _gcry_random_is_faked()
 {
@@ -213,10 +228,16 @@ static byte *
 get_random_bytes( size_t nbytes, int level, int secure )
 {
     byte *buf, *p;
+    int err;
 
     if( quick_test && level > 1 )
 	level = 1;
     MASK_LEVEL(level);
+
+    err = ath_mutex_lock (&pool_lock);
+    if (err)
+      log_fatal ("failed to acquire the pool lock: %s\n", strerror (err));
+    pool_is_locked = 1;
     if( level == 1 ) {
 	rndstats.getbytes1 += nbytes;
 	rndstats.ngetbytes1++;
@@ -234,24 +255,112 @@ get_random_bytes( size_t nbytes, int level, int secure )
 	nbytes -= n;
 	p += n;
     }
+
+    pool_is_locked = 0;
+    err = ath_mutex_unlock (&pool_lock);
+    if (err)
+      log_fatal ("failed to release the pool lock: %s\n", strerror (err));
     return buf;
 }
 
 void *
 gcry_random_bytes( size_t nbytes, enum gcry_random_level level )
 {
-    return get_random_bytes( nbytes, level, 0 );
+  if (!is_initialized)
+    initialize();
+  return get_random_bytes( nbytes, level, 0 );
 }
 
 void *
 gcry_random_bytes_secure( size_t nbytes, enum gcry_random_level level )
 {
-    return get_random_bytes( nbytes, level, 1 );
+  if (!is_initialized)
+    initialize();
+  return get_random_bytes( nbytes, level, 1 );
 }
 
 
-/****************
- * Mix the pool
+/* Fill the buffer with LENGTH bytes of cryptographically strong
+   random bytes. level 0 is not very strong, 1 is strong enough for
+   most usage, 2 is good for key generation stuff but may be very
+   slow.  */
+void
+gcry_randomize (byte *buffer, size_t length, enum gcry_random_level level)
+{
+  byte *p;
+  int err;
+
+  if (!is_initialized)
+    initialize ();
+
+  if( quick_test && level > 1 )
+    level = 1;
+  MASK_LEVEL(level);
+
+  err = ath_mutex_lock (&pool_lock);
+  if (err)
+    log_fatal ("failed to acquire the pool lock: %s\n", strerror (err));
+  pool_is_locked = 1;
+  if (level == 1)
+    {
+      rndstats.getbytes1 += length;
+      rndstats.ngetbytes1++;
+    }
+  else if (level >= 2)
+    {
+      rndstats.getbytes2 += length;
+      rndstats.ngetbytes2++;
+    }
+
+  for (p = buffer; length > 0;)
+    {
+      size_t n = length > POOLSIZE? POOLSIZE : length;
+      read_pool (p, n, level);
+      length -= n;
+      p += n;
+    }
+
+  pool_is_locked = 0;
+  err = ath_mutex_unlock (&pool_lock);
+  if (err)
+    log_fatal ("failed to release the pool lock: %s\n", strerror (err));
+}
+
+
+
+
+/*
+   Mix the pool:
+
+   |........blocks*20byte........|20byte|..44byte..|
+   <..44byte..>           <20byte> 
+        |                    |
+        |                    +------+
+        +---------------------------|----------+
+                                    v          v
+   |........blocks*20byte........|20byte|..44byte..|
+                                 <.....64bytes.....>   
+                                         |
+      +----------------------------------+
+     Hash
+      v
+   |.............................|20byte|..44byte..|
+   <20byte><20byte><..44byte..>
+      |                |
+      |                +---------------------+
+      +-----------------------------+        |
+                                    v        v
+   |.............................|20byte|..44byte..|
+                                 <.....64byte......>
+                                        |
+              +-------------------------+
+             Hash
+              v
+   |.............................|20byte|..44byte..|
+   <20byte><20byte><..44byte..>
+
+   and so on until we did this for all blocks. 
+
  */
 static void
 mix_pool(byte *pool)
@@ -261,16 +370,22 @@ mix_pool(byte *pool)
     int i, n;
     RMD160_CONTEXT md;
 
+    assert (pool_is_locked);
     _gcry_rmd160_init( &md );
- #if DIGESTLEN != 20
-    #error must have a digest length of 20 for ripe-md-160
- #endif
+#if DIGESTLEN != 20
+#  error must have a digest length of 20 for ripe-md-160
+#endif
     /* loop over the pool */
     pend = pool + POOLSIZE;
     memcpy(hashbuf, pend - DIGESTLEN, DIGESTLEN );
     memcpy(hashbuf+DIGESTLEN, pool, BLOCKLEN-DIGESTLEN);
     _gcry_rmd160_mixblock( &md, hashbuf);
     memcpy(pool, hashbuf, 20 );
+    if (failsafe_digest_valid && (char *)pool == rndpool)
+      {
+        for (i=0; i < 20; i++)
+          pool[i] ^= failsafe_digest[i];
+      }
 
     p = pool;
     for( n=1; n < POOLBLOCKS; n++ ) {
@@ -291,7 +406,16 @@ mix_pool(byte *pool)
 	_gcry_rmd160_mixblock( &md, hashbuf);
 	memcpy(p, hashbuf, 20 );
     }
-    burn_stack (200); /* for the rmd160_mixblock() */
+    /* Hmmm: our hash implementation does only leave small parts (64
+       bytes) of the pool on the stack, so I thnik it ios okay not to
+       require secure memory here.  Before we use this pool, it gets
+       copied to the help buffer anyway. */
+    if ( (char*)pool == rndpool)
+      {
+        _gcry_rmd160_hash_buffer (failsafe_digest, pool, POOLSIZE);
+        failsafe_digest_valid = 1;
+      }
+    burn_stack (384); /* for the rmd160_mixblock(), rmd160_hash_buffer */
 }
 
 void
@@ -314,6 +438,7 @@ read_seed_file()
     unsigned char buffer[POOLSIZE];
     int n;
 
+    assert (pool_is_locked);
     if( !seed_file_name )
 	return 0;
 
@@ -388,43 +513,56 @@ read_seed_file()
 void
 _gcry_update_random_seed_file()
 {
-    ulong *sp, *dp;
-    int fd, i;
-
-    if( !seed_file_name || !is_initialized || !pool_filled )
-	return;
-    if( !allow_seed_file_update ) {
-	log_info(_("note: random_seed file not updated\n"));
-	return;
+  ulong *sp, *dp;
+  int fd, i;
+  int err;
+  
+  if ( !seed_file_name || !is_initialized || !pool_filled )
+    return;
+  if ( !allow_seed_file_update )
+    {
+      log_info(_("note: random_seed file not updated\n"));
+      return;
     }
 
+  err = ath_mutex_lock (&pool_lock);
+  if (err)
+    log_fatal ("failed to acquire the pool lock: %s\n", strerror (err));
+  pool_is_locked = 1;
 
     /* copy the entropy pool to a scratch pool and mix both of them */
-    for(i=0,dp=(ulong*)keypool, sp=(ulong*)rndpool;
-				    i < POOLWORDS; i++, dp++, sp++ ) {
-	*dp = *sp + ADD_VALUE;
+  for (i=0,dp=(ulong*)keypool, sp=(ulong*)rndpool;
+       i < POOLWORDS; i++, dp++, sp++ ) 
+    {
+      *dp = *sp + ADD_VALUE;
     }
-    mix_pool(rndpool); rndstats.mixrnd++;
-    mix_pool(keypool); rndstats.mixkey++;
+  mix_pool(rndpool); rndstats.mixrnd++;
+  mix_pool(keypool); rndstats.mixkey++;
 
-  #ifdef HAVE_DOSISH_SYSTEM
-    fd = open( seed_file_name, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,
-							S_IRUSR|S_IWUSR );
-  #else
-    fd = open( seed_file_name, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR );
-  #endif
-    if( fd == -1 ) {
-	log_info(_("can't create `%s': %s\n"), seed_file_name, strerror(errno) );
-	return;
+#ifdef HAVE_DOSISH_SYSTEM
+  fd = open (seed_file_name, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,
+             S_IRUSR|S_IWUSR );
+#else
+  fd = open (seed_file_name, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR );
+#endif
+
+  if (fd == -1 )
+    log_info (_("can't create `%s': %s\n"), seed_file_name, strerror(errno) );
+  else 
+    {
+      do {
+	i = write (fd, keypool, POOLSIZE );
+      } while( i == -1 && errno == EINTR );
+    if (i != POOLSIZE) 
+      log_info(_("can't write `%s': %s\n"), seed_file_name, strerror(errno) );
+    if (close(fd))
+      log_info(_("can't close `%s': %s\n"), seed_file_name, strerror(errno) );
     }
-    do {
-	i = write( fd, keypool, POOLSIZE );
-    } while( i == -1 && errno == EINTR );
-    if( i != POOLSIZE ) {
-	log_info(_("can't write `%s': %s\n"), seed_file_name, strerror(errno) );
-    }
-    if( close(fd) )
-	log_info(_("can't close `%s': %s\n"), seed_file_name, strerror(errno) );
+
+  pool_is_locked = 0;
+  err = ath_mutex_unlock (&pool_lock);
+  if (err)
+    log_fatal ("failed to release the pool lock: %s\n", strerror (err));
 }
 
 
@@ -434,6 +572,7 @@ read_pool( byte *buffer, size_t length, int level )
     int i;
     ulong *sp, *dp;
 
+    assert (pool_is_locked);
     if( length > POOLSIZE ) {
 	log_bug("too many random bits requested\n");
     }
@@ -443,7 +582,7 @@ read_pool( byte *buffer, size_t length, int level )
 	    pool_filled = 1;
     }
 
-    /* For level 2 quality (key generation) we alwas make
+    /* For level 2 quality (key generation) we always make
      * sure that the pool has been seeded enough initially */
     if( level == 2 && !did_initial_extra_seeding ) {
 	size_t needed;
@@ -476,8 +615,8 @@ read_pool( byte *buffer, size_t length, int level )
     while( !pool_filled )
 	random_poll();
 
-    /* do always a fast random poll */
-    fast_random_poll();
+    /* always do a fast random poll - we have to use the unlocked version*/
+    do_fast_random_poll();
 
     if( !level ) { /* no need for cryptographic strong random */
 	/* create a new pool */
@@ -534,12 +673,13 @@ add_randomness( const void *buffer, size_t length, int source )
 {
     const byte *p = buffer;
 
+    assert (pool_is_locked);
     if( !is_initialized )
 	initialize();
     rndstats.addbytes += length;
     rndstats.naddbytes++;
     while( length-- ) {
-	rndpool[pool_writepos++] = *p++;
+	rndpool[pool_writepos++] ^= *p++;
 	if( pool_writepos >= POOLSIZE ) {
 	    if( source > 1 )
 		pool_filled = 1;
@@ -560,12 +700,14 @@ random_poll()
 }
 
 
-void
-_gcry_fast_random_poll()
+
+static void
+do_fast_random_poll ()
 {
     static void (*fnc)( void (*)(const void*, size_t, int), int) = NULL;
     static int initialized = 0;
 
+    assert (pool_is_locked);
     rndstats.fastpolls++;
     if( !initialized ) {
 	if( !is_initialized )
@@ -637,6 +779,32 @@ _gcry_fast_random_poll()
 }
 
 
+void
+_gcry_fast_random_poll()
+{
+  int err;
+
+  /* We have to make sure that the intialization is done because this
+     gatherer might be called before any other functions and it is not
+     sufficient to initialize it within do_fast_random_pool becuase we
+     want to use the mutex here. FIXME: Whe should initialize the mutex
+     using a global constructor independent from the initialization
+     of the pool. */
+  if (!is_initialized)
+    initialize ();
+  err = ath_mutex_lock (&pool_lock);
+  if (err)
+    log_fatal ("failed to acquire the pool lock: %s\n", strerror (err));
+  pool_is_locked = 1;
+  do_fast_random_poll ();
+  pool_is_locked = 0;
+  err = ath_mutex_unlock (&pool_lock);
+  if (err)
+    log_fatal ("failed to acquire the pool lock: %s\n", strerror (err));
+
+}
+
+
 
 static void
 read_random_source( int requester, size_t length, int level )
@@ -672,31 +840,28 @@ gather_faked( void (*add)(const void*, size_t, int), int requester,
 	/* we can't use tty_printf here - do we need this function at
 	  all - does it really make sense or canit be viewed as a potential
 	  security problem ? wk 17.11.99 */
-#if __GNUC__ >= 2
-#     warning Extended warning disabled
-#endif
-      #if 0
+#if 0
 	tty_printf(_("The random number generator is only a kludge to let\n"
 		   "it run - it is in no way a strong RNG!\n\n"
 		   "DON'T USE ANY DATA GENERATED BY THIS PROGRAM!!\n\n"));
-      #endif
+#endif
 	initialized=1;
-      #ifdef HAVE_RAND
+#ifdef HAVE_RAND
 	srand( time(NULL)*getpid());
-      #else
+#else
 	srandom( time(NULL)*getpid());
-      #endif
+#endif
     }
 
     p = buffer = gcry_xmalloc( length );
     n = length;
-  #ifdef HAVE_RAND
+#ifdef HAVE_RAND
     while( n-- )
 	*p++ = ((unsigned)(1 + (int) (256.0*rand()/(RAND_MAX+1.0)))-1);
-  #else
+#else
     while( n-- )
 	*p++ = ((unsigned)(1 + (int) (256.0*random()/(RAND_MAX+1.0)))-1);
-  #endif
+#endif
     add_randomness( buffer, length, requester );
     gcry_free(buffer);
     return 0; /* okay */
