@@ -1,23 +1,9 @@
 /* rndw32.c  -	W32 entropy gatherer
- *	Copyright (C) 1999, 2000 Free Software Foundation, Inc.
+ *	Copyright (C) 1999, 2000, 2002 Free Software Foundation, Inc.
  *	Copyright Peter Gutmann, Matt Thomlinson and Blake Coverett 1996-1999
  *
  * This file is part of Libgcrypt.
  *
- * Libgcrypt is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * Libgcrypt is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- *
  *************************************************************************
  * The code here is based on code from Cryptlib 3.0 beta by Peter Gutmann.
  * Source file misc/rndwin32.c "Win32 Randomness-Gathering Code" with this
@@ -82,7 +68,7 @@
 #ifdef IS_MODULE
   #define _(a) (a)
 #else
-  #include "i18n.h"
+/*#include "i18n.h"*/
 #endif
 
 
@@ -191,7 +177,7 @@ load_and_init_winseed( void )
     /*log_info("fast buffer size=%u\n", n2);*/
 
     entropy_buffer_size =  n1 > n2? n1: n2;
-    entropy_buffer = m_alloc( entropy_buffer_size );
+    entropy_buffer = gcry_xmalloc( entropy_buffer_size );
     /*log_info("using a buffer of size=%u\n", entropy_buffer_size );*/
 
     return;
@@ -249,8 +235,8 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	    set_internal_seed_size( slow_seeder, n1 );
 	    if( n1 > entropy_buffer_size ) {
 		entropy_buffer_size =  n1;
-		entropy_buffer = m_realloc( entropy_buffer,
-					    entropy_buffer_size );
+		entropy_buffer = gcry_realloc( entropy_buffer,
+                                               entropy_buffer_size );
 	    }
 	    continue;
 	}
@@ -690,7 +676,7 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
      * worst-case estimate which is usually nowhere near the actual amount
      * required.  For example it may report that 128K of memory is required,
      * but only return 64K of data */
-    {	pPerfData =  m_alloc (cbPerfData);
+    {	pPerfData =  gcry_xmalloc (cbPerfData);
 	for (;;) {
 	    dwSize = cbPerfData;
 	    if ( debug_me )
@@ -707,14 +693,14 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
 	    }
 	    else if (status == ERROR_MORE_DATA) {
 		cbPerfData += PERFORMANCE_BUFFER_STEP;
-		pPerfData = m_realloc (pPerfData, cbPerfData);
+		pPerfData = gcry_realloc (pPerfData, cbPerfData);
 	    }
 	    else {
 		log_debug ( "rndw32: get performance data problem\n");
 		break;
 	    }
 	}
-	m_free (pPerfData);
+	gcry_free (pPerfData);
     }
     /* Although this isn't documented in the Win32 API docs, it's necessary
        to explicitly close the HKEY_PERFORMANCE_DATA key after use (it's
@@ -753,7 +739,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
                         || (is_windowsNT && osvi.dwMajorVersion >= 5));
 
 	if ( platform == VER_PLATFORM_WIN32s ) {
-	    g10_log_fatal("can't run on a W32s platform\n" );
+	    log_fatal("can't run on a W32s platform\n" );
 	}
 	is_initialized = 1;
 	if ( debug_me )
@@ -917,7 +903,7 @@ gather_random_fast( void (*add)(const void*, size_t, int), int requester )
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDW32 ($Revision: 1.8 $)";
+const char * const gnupgext_version = "RNDW32 ($Revision: 1.11 $)";
 
 static struct {
     int class;
@@ -956,10 +942,10 @@ gnupgext_enum_func( int what, int *sequence, int *class, int *vers )
 
 #ifndef IS_MODULE
 void
-rndw32_constructor(void)
+_gcry_rndw32_constructor(void)
 {
-    register_internal_cipher_extension( gnupgext_version,
-					gnupgext_enum_func );
+  _gcry_register_internal_cipher_extension( gnupgext_version,
+                                            gnupgext_enum_func );
 }
 #endif
 
