@@ -1,20 +1,20 @@
 /* tiger.c  -  The TIGER hash function
- *	Copyright (C) 1998, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
  * Libgcrypt is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
  *
  * Libgcrypt is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
@@ -813,12 +813,12 @@ tiger_final( TIGER_CONTEXT *hd )
 
     p = hd->buf;
   #ifdef BIG_ENDIAN_HOST
-    #define X(a) do { *(u64*)p = hd->##a ; p += 8; } while(0)
+    #define X(a) do { *(u64*)p = hd->a ; p += 8; } while(0)
   #else /* little endian */
-    #define X(a) do { *p++ = hd->##a >> 56; *p++ = hd->##a >> 48; \
-		      *p++ = hd->##a >> 40; *p++ = hd->##a >> 32; \
-		      *p++ = hd->##a >> 24; *p++ = hd->##a >> 16; \
-		      *p++ = hd->##a >>  8; *p++ = hd->##a; } while(0)
+    #define X(a) do { *p++ = hd->a >> 56; *p++ = hd->a >> 48; \
+		      *p++ = hd->a >> 40; *p++ = hd->a >> 32; \
+		      *p++ = hd->a >> 24; *p++ = hd->a >> 16; \
+		      *p++ = hd->a >>  8; *p++ = hd->a; } while(0)
   #endif
     X(a);
     X(b);
@@ -847,20 +847,10 @@ tiger_get_info( int algo, size_t *contextsize,
 	       byte *(**r_read)( void *c )
 	     )
 {
-    /* 40: SEQUENCE {
-     * 12:   SEQUENCE {
-     *	8:     OCTET STRING   :54 49 47 45 52 31 39 32
-     *	0:     NULL
-     *	 :     }
-     * 24:   OCTET STRING
-     *	 :   }
-     *
-     * By replacing the 5th byte (0x04) with 0x16 we would have;
-     *	      8:     IA5String 'TIGER192'
-     */
-    static byte asn[18] =
-		{ 0x30, 0x28, 0x30, 0x0c, 0x04, 0x08, 0x54, 0x49, 0x47,
-		  0x45, 0x52, 0x31, 0x39, 0x32, 0x05, 0x00, 0x04, 0x18 };
+    static byte asn[19] = /* Object ID is 1.3.6.1.4.1.11591.12.2 */
+                         { 0x30, 0x29, 0x30, 0x0d, 0x06, 0x09, 0x2b, 0x06,
+                           0x01, 0x04, 0x01, 0xda, 0x47, 0x0c, 0x02,
+                           0x05, 0x00, 0x04, 0x18 };
 
     if( algo != 6 )
 	return NULL;
@@ -882,7 +872,7 @@ tiger_get_info( int algo, size_t *contextsize,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "TIGER ($Revision: 1.15 $)";
+const char * const gnupgext_version = "TIGER ($Revision: 1.19 $)";
 
 static struct {
     int class;
@@ -949,13 +939,12 @@ gnupgext_enum_func( int what, int *sequence, int *class, int *vers )
 
 #ifndef IS_MODULE
 void
-tiger_constructor(void)
+_gcry_tiger_constructor(void)
 {
-    register_internal_cipher_extension( gnupgext_version,
-					gnupgext_enum_func );
+  _gcry_register_internal_cipher_extension( gnupgext_version,
+                                            gnupgext_enum_func );
 }
 #endif
 
 
 #endif /* HAVE_U64_TYPEDEF */
-
